/*
Copyright (c) 2010, Stephane Duguay < s (@) binarez (.) com >
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided
that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and
  the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this list of conditions
  and the following disclaimer in the documentation and/or other materials provided with the distribution.
* Neither the name of the <ORGANIZATION> nor the names of its contributors may be used to endorse or promote
  products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#include "touchoptions.h"
#include "getopt.h"

//------------------------------------------------------------------------------------------------//
TouchOptions::TouchOptions( int argc, wchar_t* argv[], int& rFilesIndex )
: mChangeAccessTime( false )
, mChangeModifTime( false )
, mChangeCreationTime( false )
, mNoCreate( false )
, mReferenceFile( L"" )
, mTimestamp( L"" )
, mHelp( false )
, mVersion( false )
, mRecursive( false )
, mVerbose( false )
, mPauseOnExit( false )
{
	static const int kVersionTag = 0x5ca1ab1e;
	static const struct option longOptions[] =
	{
		{ L"access-time", no_argument, NULL, 'a' },
		{ L"date", required_argument, NULL, 'd' },
		{ L"help", no_argument, NULL, 'h' },
		{ L"modif-time", no_argument, NULL, 'm' },
		{ L"pause-exit", no_argument, NULL, 'p' },
		{ L"reference", required_argument, NULL, 'r' },
		{ L"no-create", no_argument, NULL, 'c' },
		{ L"recursive", no_argument, NULL, 's' },
		{ L"time", required_argument, NULL, 't' },
		{ L"verbose", no_argument, NULL, 'v' },
		{ L"version", no_argument, NULL, kVersionTag },
		{ L"creation-time", no_argument, NULL, 'x' },
		{ NULL, 0, NULL, 0 }
	};
	int opt = 0;
	while( ( opt = ::getopt_long( argc, argv, L"acd:hmpr:Rst:vx", longOptions, NULL ) ) != -1 )
	{
		switch( opt )
		{
			case 'a':
				mChangeAccessTime = true;
				break;
			case 'c':
				mNoCreate = true;
				break;
			case 'd':
				mDateTime = ( optarg != NULL ) ? optarg : L"";
				break;
			case 'm':
				mChangeModifTime = true;
				break;
			case 'p':
				mPauseOnExit = true;
				break;
			case 'v':
				mVerbose = true;
				break;
			case 'r':
				mReferenceFile = ( optarg != NULL ) ? optarg : L"";
				break;
			case 'R':
			case 's':
				mRecursive = true;
				break;
			case 't':
				mTimestamp = ( optarg != NULL ) ? optarg : L"";
				break;
			case 'h':
				mHelp = true;
				break;
			case 'x':
				mChangeCreationTime = true;
				break;
			case kVersionTag:
				mVersion = true;
				break;
			case ':':
			case '?':
			default:
				break;
		}
	}

	// If all are false, we want to do both access and modification times.
	// This is default touch behavior on POSIX.
	if( !mChangeAccessTime && !mChangeModifTime && !mChangeCreationTime )
	{
		mChangeAccessTime = true;
		mChangeModifTime = true;
	}

	rFilesIndex = optind;
}
//------------------------------------------------------------------------------------------------//
TouchOptions::~TouchOptions( void )
{
}
//------------------------------------------------------------------------------------------------//
