//------------------------------------------------------------------------------------------------//
/*
Copyright (c) 2010, Stephane Duguay < s (@) binarez (.) com >
All rights reserved.

Redistribution and use in source and binary forms, with or without modification, are permitted provided
that the following conditions are met:

* Redistributions of source code must retain the above copyright notice, this list of conditions and
the following disclaimer.
* Redistributions in binary form must reproduce the above copyright notice, this list of conditions
and the following disclaimer in the documentation and/or other materials provided with the distribution.
* Neither the name of the <ORGANIZATION> nor the names of its contributors may be used to endorse or promote
products derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED
WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR
TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef TOUCH_EXE_TOUCH_UTILS_H
#define TOUCH_EXE_TOUCH_UTILS_H

#include <string>
#include <algorithm>

namespace TouchUtils
{
	//--------------------------------------------------------------------------------------------//
	inline bool containsSearchCharacters( const std::wstring& toScan )
	{
		return	( toScan.find( '*' ) != std::wstring::npos ) ||
				( toScan.find( '?' ) != std::wstring::npos );
	}
	//--------------------------------------------------------------------------------------------//
	struct ePathType
	{	enum Enum	{	Invalid = 0,
						File,
						Directory,
						Search	};
	};
	//--------------------------------------------------------------------------------------------//
	inline ePathType::Enum getPathType( const std::wstring& path )
	{
		if( containsSearchCharacters( path ) )
			{ return ePathType::Search; }

		const DWORD fileAttribute = GetFileAttributes( path.c_str() );
		if( fileAttribute != INVALID_FILE_ATTRIBUTES )
		{
			if( fileAttribute & FILE_ATTRIBUTE_DIRECTORY )
				{ return ePathType::Directory; }
			else
				{ return ePathType::File; }
		}

		return ePathType::Invalid;
	}
	//--------------------------------------------------------------------------------------------//
	inline void removeTrailingSlashes( std::wstring& toRemoveFrom ) 
	{
		std::replace( toRemoveFrom.begin(), toRemoveFrom.end(), '/', '\\' );
		while(	!toRemoveFrom.empty() && *(toRemoveFrom.rbegin()) == '\\' )
		{
			toRemoveFrom.resize( toRemoveFrom.length() - 1 );
		}
	}
	//--------------------------------------------------------------------------------------------//
	inline bool tripleLogicalXOR( bool a, bool b, bool c )
	{
		return	(a	?	!(b || c)
					:	 (b || c) && !(b && c));
	}
	//------------------------------------------------------------------------------------------------//
}

#endif // TOUCH_EXE_TOUCH_UTILS_H
